@@ -13,21 +13,21 @@ class ProtobufSerializer(SerializerAbstract):
                                        'battery': '', 'speed': '', 'course': '', 'ptpCallsigns': ''}
 
     def from_format_to_fts_object(self, object: type, FTSObject: Event) -> Event:
+        if hasattr(FTSObject.detail, 'marti'):
+            from FreeTAKServer.model.FTSModel.Dest import Dest
+
+            for callsign in object.event.ptpCallsigns:
+                newdest = Dest()
+                newdest.setcallsign(callsign)
+                FTSObject.detail.marti.setdest(newdest)
+
         for descriptor in object.event.DESCRIPTOR.fields:
             attribute_name = descriptor.name
 
             if attribute_name in self.attribute_name_mapping:
                 attribute_name = self.attribute_name_mapping[attribute_name]
 
-            if hasattr(FTSObject.detail, 'marti'):
-                from FreeTAKServer.model.FTSModel.Dest import Dest
-
-                for callsign in object.event.ptpCallsigns:
-                    newdest = Dest()
-                    newdest.setcallsign(callsign)
-                    FTSObject.detail.marti.setdest(newdest)
-
-            elif attribute_name != 'other' and attribute_name != '':
+            if attribute_name != 'other' and attribute_name != '':
 
                 setters = self._get_fts_object_var_setter(FTSObject, attribute_name)
                 setter = self._get_method_in_method_list(setters, 'event')
@@ -50,11 +50,12 @@ class ProtobufSerializer(SerializerAbstract):
 
             elif attribute_name == 'other' and object.event.other != '':
                 from defusedxml import ElementTree as etree
-                from lxml.etree import Element
+                from xml.etree.ElementTree import Element
                 xmldetail = etree.fromstring(object.event.other)
-                xmldetail.remove(xmldetail.find('_flow-tags_'))
+                if xmldetail.find('_flow-tags_'):
+                    xmldetail.remove(xmldetail.find('_flow-tags_'))
 
-                if xmldetail.find('remarks'):
+                if xmldetail.find('remarks') != None:
                     XmlSerializer().from_format_to_fts_object(etree.tostring(xmldetail).decode(), FTSObject.detail)
                 else:
                     xmldetail.append(Element('remarks'))
@@ -69,7 +70,7 @@ class ProtobufSerializer(SerializerAbstract):
             if hasattr(FTSObject.detail, 'marti'):
                 for dest in FTSObject.detail.marti.dest:
                     callsign = dest.getcallsign()
-                    if callsign:
+                    if callsign and isinstance(callsign, str):
                         obj.event.ptpCallsigns.append(callsign)
                     else:
                         pass
@@ -81,21 +82,32 @@ class ProtobufSerializer(SerializerAbstract):
                     attribute_name = self.attribute_name_mapping[attribute_name]
 
                 if attribute_name != 'other' and attribute_name:
-                    getters = self._get_fts_object_var_getter(FTSObject, attribute_name)
-                    getter = self._get_method_in_method_list(getters, 'event')
-
-                    if attribute_name == 'time' or attribute_name == 'start' or attribute_name == 'stale':
-                        import datetime
-                        attribute = getter()
-                        setattr(obj.event, descriptor.name,
-                                int(datetime.datetime.strptime(attribute, "%Y-%m-%dT%H:%M:%S.%fZ").timestamp() * 1000))
-                    elif attribute_name in ['lat', 'lon', 'hae', 'ce', 'le', 'speed', 'course']:
-                        setattr(obj.event, descriptor.name, float(getter()))
-                    elif attribute_name == 'battery':
-                        setattr(obj.event, descriptor.name, int(getter()))
-                    else:
-                        setattr(obj.event, descriptor.name, getter())
+                    try:
+                        getters = self._get_fts_object_var_getter(FTSObject, attribute_name)
+                        getter = self._get_method_in_method_list(getters, 'event')
 
+                        if attribute_name == 'time' or attribute_name == 'start' or attribute_name == 'stale':
+                            import datetime
+                            attribute = getter()
+                            setattr(obj.event, descriptor.name,
+                                    int(datetime.datetime.strptime(attribute, "%Y-%m-%dT%H:%M:%S.%fZ").timestamp() * 1000))
+                        elif attribute_name in ['lat', 'lon', 'hae', 'ce', 'le', 'speed', 'course']:
+                            setattr(obj.event, descriptor.name, float(getter()))
+                        elif attribute_name == 'battery':
+                            setattr(obj.event, descriptor.name, int(getter()))
+                        else:
+                            setattr(obj.event, descriptor.name, getter())
+                    except Exception as e:
+                        import traceback
+                        import sys
+                        import linecache
+                        exc_type, exc_obj, tb = sys.exc_info()
+                        f = tb.tb_frame
+                        lineno = tb.tb_lineno
+                        filename = f.f_code.co_filename
+                        linecache.checkcache(filename)
+                        line = linecache.getline(filename, lineno, f.f_globals)
+                        print(line)
                 else:
                     from defusedxml import ElementTree as etree
                     xmldetail = etree.tostring(XmlSerializer().from_fts_object_to_format(FTSObject.detail)).decode()
@@ -104,6 +116,7 @@ class ProtobufSerializer(SerializerAbstract):
             return obj
         except Exception as e:
             raise e
+
     def _get_method_in_method_list(self, method_list: List[callable], expected_class_name: str) -> callable:
         if len(method_list) == 1:
             return method_list[0]
