@@ -19,7 +19,10 @@ from FreeTAKServer.controllers.services.RestAPI import RestAPI
 from FreeTAKServer.model.ServiceObjects.FTS import FTS as FTSObj
 from FreeTAKServer.model.SimpleClient import SimpleClient
 from FreeTAKServer.controllers.configuration_wizard import ask_user_for_config
+
 import time
+import queue
+
 from FreeTAKServer.controllers.AddDataToCoTList import AddDataToCoTList
 from FreeTAKServer.model.FilterGroup import FilterGroup
 from FreeTAKServer.controllers.services.SSLCoTServiceController import SSLCoTServiceController
@@ -30,6 +33,8 @@ from FreeTAKServer.controllers.DatabaseControllers.DatabaseController import Dat
 from FreeTAKServer.controllers.certificate_generation import AtakOfTheCerts
 from multiprocessing import Queue
 from FreeTAKServer.controllers.configuration.MainConfig import MainConfig
+
+from FreeTAKServer.model.Enumerations.serviceTypes import ServiceTypes
 from FreeTAKServer.model.SpecificCoT.Presence import Presence
 from FreeTAKServer.model.Connection import Connection
 
@@ -45,6 +50,10 @@ loggingConstants = LoggingConstants()
 class FTS:
 
     def __init__(self):
+        self.FederationClientService = None
+        self.FederationServerService = None
+        self.SSLCoTService = None
+        self.SSLDataPackageService = None
         self.CoTService = None
         self.TCPDataPackageService = None
         self.UserCommand = None
@@ -78,7 +87,7 @@ class FTS:
             self.receive_Rest.start()
             self.RestAPIProcess.start()
             self.pipeList['restAPI'] = self.RestAPIPipe
-            self.FilterGroup.sources.append(self.RestAPIPipe)
+            self.FilterGroup.add_source(self.RestAPIPipe, ServiceTypes.APISERVICE)
             return 1
         except Exception as e:
             logger.error('There has been an exception thrown in the startup of the restAPI service ' + str(e))
@@ -111,8 +120,8 @@ class FTS:
             self.ReceiveConnectionsReset, TCPCoTService, self.core_tcp_user_queue_send))
             self.CoTService.start()
             self.pipeList['TCPCoTServiceFTSPipe'] = self.TCPCoTService
-            self.FilterGroup.receivers.append(self.TCPCoTService)
-            self.FilterGroup.sources.append(self.TCPCoTService)
+            self.FilterGroup.add_receiver(self.TCPCoTService, ServiceTypes.TCPCOTSERVICE)
+            self.FilterGroup.add_source(self.TCPCoTService, ServiceTypes.TCPCOTSERVICE)
             print('CoTService started')
             return 1
         except Exception as e:
@@ -131,8 +140,8 @@ class FTS:
             else:
                 self.CoTService.join()
 
-            self.FilterGroup.sources.remove(self.TCPCoTService)
-            self.FilterGroup.receivers.remove(self.TCPCoTService)
+            self.FilterGroup.remove_source(ServiceTypes.TCPCOTSERVICE)
+            self.FilterGroup.remove_receiver(ServiceTypes.TCPCOTSERVICE)
 
         except Exception as e:
             logger.error("there's been an exception in the stopping of CoT Service " + str(e))
@@ -151,7 +160,7 @@ class FTS:
             print('starting now')
             self.TCPDataPackageService.start()
             self.pipeList['tcp_data_package_service_pipe'] = self.tcp_data_package_service_pipe
-            self.FilterGroup.sources.append(self.tcp_data_package_service_pipe)
+            self.FilterGroup.add_source(self.tcp_data_package_service_pipe, ServiceTypes.TCPDPSERVICE)
             time.sleep(2)
             return 1
         except Exception as e:
@@ -161,7 +170,7 @@ class FTS:
 
     def stop_tcp_data_package_service(self):
         del self.pipeList['tcp_data_package_service_pipe']
-        self.FilterGroup.sources.remove(self.tcp_data_package_service_pipe)
+        self.FilterGroup.remove_source(ServiceTypes.TCPDPSERVICE)
         try:
             self.TCPDataPackageService.terminate()
         except Exception as e:
@@ -187,7 +196,7 @@ class FTS:
             print('starting SSL now')
             self.SSLDataPackageService.start()
             self.pipeList['ssl_data_package_service'] = self.ssl_data_package_service
-            self.FilterGroup.sources.append(self.ssl_data_package_service)
+            self.FilterGroup.add_source(self.ssl_data_package_service, ServiceTypes.SSLDPSERVICE)
             time.sleep(2)
             return 1
         except Exception as e:
@@ -197,7 +206,7 @@ class FTS:
 
     def stop_ssl_data_package_service(self):
         del (self.pipeList['ssl_data_package_service'])
-        self.FilterGroup.sources.remove(self.ssl_data_package_service)
+        self.FilterGroup.remove_source(ServiceTypes.SSLDPSERVICE)
         try:
             self.SSLDataPackageService.terminate()
         except Exception as e:
@@ -228,8 +237,8 @@ class FTS:
             self.service_ssl_user_queue_send, self.ReceiveConnectionsReset, SSLCoTServicePipe, self.core_ssl_user_queue_send))
             self.SSLCoTService.start()
             self.pipeList['SSLCoTServiceFTSPipe'] = self.SSLCoTServicePipe
-            self.FilterGroup.sources.append(self.SSLCoTServicePipe)
-            self.FilterGroup.receivers.append(self.SSLCoTServicePipe)
+            self.FilterGroup.add_source(self.SSLCoTServicePipe, ServiceTypes.SSLCOTSERVICE)
+            self.FilterGroup.add_receiver(self.SSLCoTServicePipe, ServiceTypes.SSLCOTSERVICE)
             print('SSL CoTService started')
             return 1
         except Exception as e:
@@ -248,8 +257,8 @@ class FTS:
             else:
                 self.SSLCoTService.join()
             del (self.pipeList["SSLCoTServiceFTSPipe"])
-            self.FilterGroup.sources.remove(self.SSLCoTServicePipe)
-            self.FilterGroup.receivers.remove(self.SSLCoTServicePipe)
+            self.FilterGroup.remove_source(ServiceTypes.SSLCOTSERVICE)
+            self.FilterGroup.remove_receiver(ServiceTypes.SSLCOTSERVICE)
 
         except Exception as e:
             logger.error("there's been an exception in the stopping of CoT Service " + str(e))
@@ -265,23 +274,21 @@ class FTS:
                                                                args=(FederationClientServicePipe,))
         self.FederationClientService.start()
         self.pipeList['FederationClientServiceFTSPipe'] = self.FederationClientServicePipeFTS
-        self.FilterGroup.sources.append(self.FederationClientServicePipeFTS)
-        self.FilterGroup.receivers.append(self.FederationClientServicePipeFTS)
+        self.FilterGroup.add_source(self.FederationClientServicePipeFTS, ServiceTypes.FEDCLIENTSERVICE)
+        self.FilterGroup.add_receiver(self.FederationClientServicePipeFTS, ServiceTypes.FEDCLIENTSERVICE)
         return 1
 
     def stop_federation_client_service(self):
         try:
             del (self.pipeList['FederationClientServiceFTSPipe'])
-            self.FilterGroup.receivers.remove(self.FederationClientServicePipeFTS)
-            self.FilterGroup.sources.remove(self.FederationClientServicePipeFTS)
             if self.FederationClientService.is_alive():
                 self.FederationClientService.terminate()
                 self.FederationClientService.join()
             else:
                 self.FederationClientService.join()
 
-            self.FilterGroup.sources.remove(self.FederationClientServicePipeFTS)
-            self.FilterGroup.receivers.remove(self.FederationClientServicePipeFTS)
+            self.FilterGroup.remove_receiver(ServiceTypes.FEDCLIENTSERVICE)
+            self.FilterGroup.remove_source(ServiceTypes.FEDCLIENTSERVICE)
             return 1
         except:
             return -1
@@ -299,10 +306,10 @@ class FTS:
                 target=FederationServerService().start, args=(FederationServerServicePipe, ip, port))
             self.FederationServerService.start()
             self.pipeList['FederationServerServiceFTSPipe'] = self.FederationServerServicePipeFTS
-            self.FilterGroup.sources.append(self.FederationServerServicePipeFTS)
-            self.FilterGroup.receivers.append(self.FederationServerServicePipeFTS)
+            self.FilterGroup.add_source(self.FederationServerServicePipeFTS, ServiceTypes.FEDSERVERSERVICE)
+            self.FilterGroup.add_receiver(self.FederationServerServicePipeFTS, ServiceTypes.FEDSERVERSERVICE)
             return 1
-        except:
+        except Exception as e:
             return -1
 
     def stop_federation_server_service(self):
@@ -313,8 +320,8 @@ class FTS:
             else:
                 self.FederationServerService.join()
             self.FederationServerServicePipeFTS.close()
-            self.FilterGroup.sources.remove(self.FederationServerServicePipeFTS)
-            self.FilterGroup.receivers.remove(self.FederationServerServicePipeFTS)
+            self.FilterGroup.remove_source(ServiceTypes.FEDSERVERSERVICE)
+            self.FilterGroup.remove_receiver(ServiceTypes.FEDSERVERSERVICE)
             return 1
         except:
             return -1
@@ -325,6 +332,8 @@ class FTS:
         try:
             if FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServiceStatus == 'start':
                 self.FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServiceStatus = FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServiceStatus
+                if isinstance(self.TCPDataPackageService, multiprocessing.Process) and self.TCPDataPackageService.is_alive(): # stop the running service and restart, this applies primarily to port changes
+                    self.stop_tcp_data_package_service()
                 self.start_tcp_data_package_service(FTSServiceStartupConfigObject)
                 if FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServicePort != "":
                     self.FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServicePort = FTSServiceStartupConfigObject.TCPDataPackageService.TCPDataPackageServicePort
@@ -346,6 +355,8 @@ class FTS:
 
             if FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServiceStatus == 'start':
                 self.FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServiceStatus = FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServiceStatus
+                if isinstance(self.SSLDataPackageService, multiprocessing.Process) and self.SSLDataPackageService.is_alive(): # stop the running service and restart, this applies primarily to port changes
+                    self.stop_ssl_data_package_service()
                 self.start_ssl_data_package_service(FTSServiceStartupConfigObject)
                 if FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServicePort != "":
                     self.FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServicePort = FTSServiceStartupConfigObject.SSLDataPackageService.SSLDataPackageServicePort
@@ -367,6 +378,8 @@ class FTS:
 
             if FTSServiceStartupConfigObject.CoTService.CoTServiceStatus == 'start':
                 self.FTSServiceStartupConfigObject.CoTService.CoTServiceStatus = FTSServiceStartupConfigObject.CoTService.CoTServiceStatus
+                if isinstance(self.CoTService, multiprocessing.Process) and self.CoTService.is_alive(): # stop the running service and restart, this applies primarily to port changes
+                    self.stop_CoT_service()
                 self.start_CoT_service(FTSServiceStartupConfigObject)
                 if FTSServiceStartupConfigObject.CoTService.CoTServicePort != "":
                     self.FTSServiceStartupConfigObject.CoTService.CoTServicePort = FTSServiceStartupConfigObject.CoTService.CoTServicePort
@@ -381,6 +394,8 @@ class FTS:
 
             if FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServiceStatus == 'start':
                 self.FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServiceStatus = FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServiceStatus
+                if isinstance(self.SSLCoTService, multiprocessing.Process) and self.SSLCoTService.is_alive(): # stop the running service and restart, this applies primarily to port changes
+                    self.stop_SSL_CoT_service()
                 self.start_SSL_CoT_service(FTSServiceStartupConfigObject)
                 if FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServicePort != "":
                     self.FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServicePort = FTSServiceStartupConfigObject.SSLCoTService.SSLCoTServicePort
@@ -396,6 +411,8 @@ class FTS:
 
             if FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceStatus == 'start':
                 self.FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceStatus = FTSServiceStartupConfigObject.FederationClientService.FederationClientServiceStatus
+                if isinstance(self.FederationClientService, multiprocessing.Process) and self.FederationClientService.is_alive(): # stop the running service and restart, this applies primarily to port changes
+                    self.stop_federation_client_service()
                 self.start_federation_client_service(FTSServiceStartupConfigObject)
                 if FTSServiceStartupConfigObject.FederationClientService.FederationClientServicePort != "":
                     self.FTSServiceStartupConfigObject.FederationClientService.FederationClientServicePort = FTSServiceStartupConfigObject.FederationClientService.FederationClientServicePort
@@ -412,6 +429,8 @@ class FTS:
 
             if FTSServiceStartupConfigObject.FederationServerService.FederationServerServiceStatus == 'start':
                 self.FTSServiceStartupConfigObject.FederationServerService.FederationServerServiceStatus = FTSServiceStartupConfigObject.FederationServerService.FederationServerServiceStatus
+                if isinstance(self.FederationServerService, multiprocessing.Process) and self.FederationServerService.is_alive(): # stop the running service and restart, this applies primarily to port changes
+                    self.stop_federation_server_service()
                 self.start_federation_server_service(FTSServiceStartupConfigObject)
                 if FTSServiceStartupConfigObject.FederationServerService.FederationServerServicePort != "":
                     self.FTSServiceStartupConfigObject.FederationServerService.FederationServerServicePort = FTSServiceStartupConfigObject.FederationServerService.FederationServerServicePort
@@ -436,12 +455,26 @@ class FTS:
         return 1
 
     def receive_data_froCoT_service_thread(self, recv_pipe, clientArray, send_pipe):
-        found = 0
+        """ this is the method centrally responsible for the managment of User information
+        it receives all client information operations from all services and updates the central
+        user_dict accordingl
+
+        Args:
+            recv_pipe: the queue to receive client information from
+            clientArray:
+            send_pipe: the queue to send updates to
+
+        Returns:
+
+        """
         # pip data should be composed of 3 parts, [operation, modelObject, opensockets, connection_object(only necessary for add opp)]
         try:
             # TODO: change 'add' 'remove' 'update' and 'get' to an enumeration
-            if not recv_pipe.empty():
-                data = recv_pipe.get()
+            try:
+                data = recv_pipe.get(timeout=MainConfig.MainLoopDelay/1000)
+            except queue.Empty:
+                return self.user_dict
+            if data:
                 self.socketCount = data[2]
 
                 if data[0] == 'add':
@@ -455,20 +488,23 @@ class FTS:
                         logger.error("error in adding client data to user_dict client presence data: " + str(data[1]) + " client connection data: " + str(data[3]))
 
                 elif data[0] == 'remove':
+                    logger.debug("removing client " + str(data[1].user_id))
                     self.user_dict[data[1].user_id].delete_connection(data[3])
-                    if len(self.user_dict[data[1].user_id].connections) <= 0:  # prevent users connected to one service being completely deleted upon disconnect
+                    if len(self.user_dict[data[1].user_id].connections) < 1:  # prevent users connected to more than one service being completely deleted upon disconnect
+                        logger.debug("client " + str(data[1].user_id) + " deleted from user_dict")
                         del self.user_dict[data[1].user_id]
+                    logger.debug('user_dict is now '+str(self.user_dict))
 
                 elif data[0] == 'update':
                     # when update is called user object is updated within the user_dict
                     if isinstance(data[1], Presence):
                         self.user_dict[data[1].modelObject.uid].m_presence = data[1]
                     else:
-                        logger.error("error in updating existing client data in user_dict client presence data: " + str(data[1]))
+                        logger.error("error in updating existing client data in user_dict client presence data: " + str(data))
 
                 elif data[0] == 'get':
                     if data[1] == None:
-                        recv_pipe.put(self.user_dict)
+                        send_pipe.put(self.user_dict)
                     else:
                         return_users = {}
                         for user_id, user_object in self.user_dict.items():
@@ -479,7 +515,6 @@ class FTS:
                 return self.user_dict
             else:
                 return self.user_dict
-
         except Exception as e:
             import traceback
             exc_type, exc_obj, tb = sys.exc_info()
@@ -488,7 +523,10 @@ class FTS:
             filename = f.f_code.co_filename
             linecache.checkcache(filename)
             line = linecache.getline(filename, lineno, f.f_globals)
-            logger.error("exception has been thrown in receive_data_froCoT_service_thread " + str(e) + " " + " error on line: "+str(line))
+            logger.error("exception has been thrown in receive_data_froCoT_service_thread " + str(e) + " " + " error on line: "+str(line) +
+                         "\n user_dict: " + str(self.user_dict) +
+                         "\n data: " + str(data) +
+                         "\n connections "+ str(self.user_dict[data[1].user_id].connections), exc_info=True)
             return self.user_dict
 
     def receive_Rest_commands(self, kill):
@@ -587,31 +625,32 @@ class FTS:
             logger.error('error in kill function ' + str(e))
 
     def checkPipes(self):
-        try:
-            for pipe in self.FilterGroup.sources:
-                try:
-                    data = AddDataToCoTList().recv(pipe, timeout=MainConfig.MainLoopDelay / 4000)
-                except Exception as e:
-                    logger.error('get pipe data failed ' + str(e))
-                    continue
-                # this runs in the event a new client has connected
-                try:
-                    if data == 0 or data is None:
+        while True:
+            try:
+                for service_name, pipe in self.FilterGroup.get_sources().items():
+                    try:
+                        data = AddDataToCoTList().recv(pipe, timeout=MainConfig.MainLoopDelay / 1000)
+                    except Exception as e:
+                        logger.error('get pipe data failed ' + str(e))
                         continue
-                    elif isinstance(data, list):
-                        AddDataToCoTList().send(self.FilterGroup.receivers, data[0])
-                        for client in self.user_dict.values():
-                            AddDataToCoTList().send(self.FilterGroup.receivers, client.m_presence) # send presence objects of all clients too the service with a new client
-                    # this runs in all other cases in which data is received
-                    elif data != 0 and data is not None:
-                        AddDataToCoTList().send(self.FilterGroup.receivers, data)
-                    # this runs when a timeout is triggered
-                    else:
-                        pass
-                except Exception as e:
-                    logger.error('processing received connection data failed ' + str(e))
-        except Exception as e:
-            logger.error('exception in checking pipes ' + str(e))
+                    # this runs in the event a new client has connected
+                    try:
+                        if data == 0 or data is None:
+                            continue
+                        elif isinstance(data, list):
+                            AddDataToCoTList().send(self.FilterGroup.receivers, data[0], service_name)
+                            for client in self.user_dict.values():
+                                AddDataToCoTList().send(self.FilterGroup.receivers, client.m_presence, service_name) # send presence objects of all clients too the service with a new client
+                        # this runs in all other cases in which data is received
+                        elif data != 0 and data is not None:
+                            AddDataToCoTList().send(self.FilterGroup.receivers, data, service_name)
+                        # this runs when a timeout is triggered
+                        else:
+                            pass
+                    except Exception as e:
+                        logger.error('processing received connection data failed ' + str(e))
+            except Exception as e:
+                logger.error('exception in checking pipes ' + str(e))
 
     def startup(self, CoTPort, CoTIP, DataPackagePort, DataPackageIP, SSLDataPackagePort, SSLDataPackageIP, RestAPIPort,
                 RestAPIIP, SSLCoTPort, SSLCoTIP, AutoStart, firstStart=False, UI="False"):
@@ -662,19 +701,14 @@ class FTS:
                 self.start_all(StartupObject)
 
             start_timer = time.time() - 60
-
+            threading.Thread(target=self.checkPipes).start()
             while True:
-                time.sleep(MainConfig.MainLoopDelay / 1000)
                 try:
                     if time.time() > start_timer+60:
                         start_timer = time.time()
                         logger.debug(str(self.user_dict))
                 except Exception as e:
                     logger.error("the periodic debug message has thrown an error "+str(e))
-                try:
-                    self.checkPipes()
-                except Exception as e:
-                    logger.error("error in core FTS process Data Pipe " + str(e))
                 try:
                     self.user_dict = self.receive_data_froCoT_service_thread(self.service_tcp_user_queue_send, self.user_dict, self.core_tcp_user_queue_send)
                 except Exception as e:
