@@ -1,6 +1,7 @@
+from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
 from FreeTAKServer.model.FTSModelVariables.CreatorUidVariables import CreatorUidVariables as vars
 
-class CreatorUid:
+class CreatorUid(FTSProtocolObject):
     def __init__(self):
         self.INTAG = None
 
