@@ -1,5 +1,6 @@
+from FreeTAKServer.model.FTSModel.fts_protocol_object import FTSProtocolObject
 from FreeTAKServer.model.FTSModelVariables.NameVariables import NameVariables as vars
-class Name:
+class Name(FTSProtocolObject):
     @staticmethod
     def ExcheckUpdate(INTAG = vars.ExcheckUpdate().INTAG):
         name = Name()
@@ -16,4 +17,4 @@ class Name:
         self.INTAG = INTAG
 
     def getINTAG(self):
-        return self.INTAG
\ No newline at end of file
+        return self.INTAG
