@@ -1,6 +1,11 @@
 class CreatorUidVariables:
     @classmethod
-    def CreateExCheckTemplate(cls):
+    def ExcheckUpdate(cls):
+        cls.INTAG = None
+        return cls
+
+    @classmethod
+    def Checklist(cls):
         cls.INTAG = None
         return cls
 
