@@ -28,11 +28,23 @@ class RoutePost(Route):
     def setlongitudeDest(self, longitudeDest):
         self.longitudeDest = str(longitudeDest)
 
-    def getname(self):
-        return self.name
+    def getrouteName(self):
+        return self.routeName
 
-    def setname(self, name):
-        self.name = name
+    def setrouteName(self, name):
+        self.routeName = name
+
+    def setstartName(self, name):
+        self.startName = name
+
+    def getstartName(self):
+        return self.startName
+
+    def setendName(self, endName):
+        self.endName = endName
+
+    def getendName(self):
+        return self.endName
 
     def gettimeout(self):
         return self.timeout
